@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-shell.h,v 1.15 2004/12/31 16:09:39 chpe Exp $
+ *  $Id: ephy-embed-shell.h,v 1.16.2.1 2005/07/04 12:14:55 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_SHELL_H
@@ -50,6 +50,11 @@ struct _EphyEmbedShell
 struct _EphyEmbedShellClass
 {
 	GObjectClass parent_class;
+
+	void	  (* prepare_close)	(EphyEmbedShell *shell);
+
+	/*< private >*/
+	GObject * (* get_embed_single)  (EphyEmbedShell *shell);
 };
 
 GType		   ephy_embed_shell_get_type		(void);
@@ -64,6 +69,8 @@ GObject		  *ephy_embed_shell_get_encodings	(EphyEmbedShell *shell);
 
 GObject		  *ephy_embed_shell_get_embed_single	(EphyEmbedShell *shell);
 
+void		   ephy_embed_shell_prepare_close	(EphyEmbedShell *shell);
+
 G_END_DECLS
 
 #endif /* !EPHY_EMBED_SHELL_H */
