@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyUtils.cpp,v 1.8.2.1 2005/04/07 11:46:32 chpe Exp $
+ *  $Id$
  */
 
 #include "mozilla-config.h"
@@ -216,3 +216,16 @@ EphyUtils::CollatePrintSettings (const EmbedPrintInfo *info,
 
 	return NS_OK;
 }
+
+PRBool
+EphyUtils::StringEquals (nsAString &s1, nsAString &s2)
+{
+	if (s1.Length() != s2.Length())
+	{
+		return FALSE;
+	}
+
+	return (memcmp (s1.BeginReading(),
+		        s2.BeginReading(),
+		        s2.Length() * sizeof (PRUnichar)) == 0);
+}
