@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.34.2.1 2005/04/07 11:46:32 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.34.2.2 2005/04/26 12:02:28 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -46,7 +46,7 @@ MozGlobalHistory::~MozGlobalHistory ()
 {
 }
 
-#ifdef MOZ_NSIGLOBALHISTORY2_ADDURI_WITH_REFERRER
+#ifdef HAVE_GECKO_1_8
 /* void addURI (in nsIURI aURI, in boolean aRedirect, in boolean aToplevel, in nsIURI aReferrer); */
 NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aToplevel, nsIURI *aReferrer)
 #else
