@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.109.2.2 2005/04/07 11:46:32 chpe Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.109.2.3 2005/07/04 12:14:56 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -28,6 +28,7 @@
 #include "ephy-cookie-manager.h"
 #include "ephy-password-manager.h"
 #include "ephy-permission-manager.h"
+#include "ephy-embed-shell.h"
 
 #include "glib.h"
 #include "ephy-debug.h"
@@ -537,6 +538,20 @@ init_services (MozillaEmbedSingle *single)
 	return TRUE;
 }
 
+static void
+prepare_close_cb (EphyEmbedShell *shell)
+{
+	GValue value = { 0, };
+
+	/* To avoid evil web sites posing an alert and thus inhibiting
+	 * shutdown, we just turn off javascript! :)
+	 */
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, FALSE);
+	mozilla_pref_set ("javascript.enabled", &value);
+	g_value_unset (&value);
+}
+
 static void
 mozilla_embed_single_init (MozillaEmbedSingle *mes)
 {
@@ -564,6 +579,10 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 
 		exit (0);
 	}
+
+	g_signal_connect_object (embed_shell, "prepare-close",
+				 G_CALLBACK (prepare_close_cb), mes,
+				 (GConnectFlags) 0);
 }
 
 static void
