@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.101.2.1 2005/04/07 11:46:32 chpe Exp $
+ *  $Id$
  */
 
 #include "mozilla-config.h"
@@ -88,12 +88,20 @@ struct MozillaEmbedPrivate
 {
 	EphyBrowser *browser;
 	MozillaEmbedLoadState load_state;
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+	guint focus_connected : 1;
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
 };
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
 static GObjectClass *parent_class = NULL;
 
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+static guint fiesid = 0;
+static guint foesid = 0;
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
+
 static void
 impl_manager_do_command (EphyCommandManager *manager,
 			 const char *command) 
@@ -207,26 +215,106 @@ impl_find_set_properties (EphyEmbed *embed,
 					   wrap_around); 
 }
 
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+static gboolean
+child_focus_in_event_cb (GtkWidget *child,
+			 GdkEventFocus *event,
+			 MozillaEmbed *embed)
+{
+	embed->priv->browser->FocusActivate ();
+
+	return FALSE;
+}
+
+static gboolean
+child_focus_out_event_cb (GtkWidget *child,
+			  GdkEventFocus *event,
+			  MozillaEmbed *embed)
+{
+	embed->priv->browser->FocusDeactivate ();
+
+	return FALSE;
+}
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
+
 static void
 mozilla_embed_realize (GtkWidget *widget)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (widget)->priv;
+	GtkBin *bin = GTK_BIN (widget);
 
-	(* GTK_WIDGET_CLASS(parent_class)->realize) (widget);
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
+	/* Initialise our helper class */
 	nsresult rv;
 	rv = mpriv->browser->Init (GTK_MOZ_EMBED (widget));
-
 	if (NS_FAILED (rv))
 	{
-               	g_warning ("EphyBrowser initialization failed for %p\n", widget);
+		g_warning ("EphyBrowser initialization failed for %p\n", widget);
+		return;
 	}
+
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+	/* HACK ALERT! This depends highly on undocumented interna of
+	 * GtkMozEmbed!
+	 *
+	 * GtkMozEmbed::realize installs focus-[in|out]-event handlers to
+	 * toplevel, and, on the first realize only, to the child.
+	 * GtkMozEmbed disconnects its focus-[in|out]-event handler
+	 * to the toplevel on unrealize, and leaves the ones to the child
+	 * in place. So we don't need to unblock the blocked handlers
+	 * and therefore need no ::unrealize handler.
+	*/
+
+	GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+	gpointer data = ((GtkMozEmbed *) widget)->data;
+
+	guint n;
+
+	n = g_signal_handlers_block_matched (toplevel,
+					     (GSignalMatchType) (G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DATA),
+					     fiesid, 0, NULL, NULL, data);
+	n += g_signal_handlers_block_matched (toplevel,
+					      (GSignalMatchType) (G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DATA),
+					      foesid, 0, NULL, NULL, data);
+	if (n != 2)
+	{
+		g_warning ("Unexpected number (n=%d) of toplevel focus handlers found!\n", n);
+	}
+
+	if (mpriv->focus_connected) return;
+
+	GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
+	g_return_if_fail (child != NULL);
+
+	n = g_signal_handlers_block_matched (child,
+					     (GSignalMatchType) (G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DATA),
+					     fiesid, 0, NULL, NULL, widget);
+	n += g_signal_handlers_block_matched (child,
+					      (GSignalMatchType) (G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DATA),
+					      foesid, 0, NULL, NULL, widget);
+	if (n != 2)
+	{
+		g_warning ("Unexpected number (n=%d) of child focus handlers found!\n", n);
+	}
+
+	g_signal_connect_object (child, "focus-in-event",
+				 G_CALLBACK (child_focus_in_event_cb), widget,
+				 G_CONNECT_AFTER);
+	g_signal_connect_object (child, "focus-out-event",
+				 G_CALLBACK (child_focus_out_event_cb), widget,
+				 G_CONNECT_AFTER);
+
+	mpriv->focus_connected = TRUE;
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
 }
 
 static GObject *
 mozilla_embed_constructor (GType type, guint n_construct_properties,
 			   GObjectConstructParam *construct_params)
 {
+	g_object_ref (embed_shell);
+
 	/* we depend on single because of mozilla initialization */
 	ephy_embed_shell_get_embed_single (embed_shell);
 
@@ -250,6 +338,11 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 
 	widget_class->realize = mozilla_embed_realize;
 
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+	fiesid = g_signal_lookup ("focus-in-event", GTK_TYPE_WIDGET);
+	foesid = g_signal_lookup ("focus-out-event", GTK_TYPE_WIDGET);
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
+
 	g_type_class_add_private (object_class, sizeof(MozillaEmbedPrivate));
 }
 
@@ -312,6 +405,8 @@ mozilla_embed_finalize (GObject *object)
 	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
+
+	g_object_unref (embed_shell);
 }
 
 static void
@@ -479,34 +574,37 @@ impl_get_location (EphyEmbed *embed,
                    gboolean toplevel)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	char *l;
 	nsresult rv;
-	nsEmbedCString url;
 
+	nsCOMPtr<nsIURI> uri;
 	if (toplevel)
 	{
-		rv = mpriv->browser->GetDocumentUrl (url);
-		l = (NS_SUCCEEDED (rv) && url.Length()) ?
-		     g_strdup (url.get()) : NULL;	   	
+		rv = mpriv->browser->GetDocumentURI (getter_AddRefs (uri));
 	}
 	else
 	{
-		rv = mpriv->browser->GetTargetDocumentUrl (url);
-		l = (NS_SUCCEEDED (rv) && url.Length()) ?
-		     g_strdup (url.get()) : NULL;	   	
+		rv = mpriv->browser->GetTargetDocumentURI (getter_AddRefs (uri));
 	}
 
-	return l;
+	if (NS_FAILED (rv)) return NULL;
+
+	nsCOMPtr<nsIURI> furi;
+	rv = uri->Clone (getter_AddRefs (furi));
+	if (NS_FAILED (rv)) return NULL;
+
+	furi->SetPassword (nsEmbedCString());
+
+	nsEmbedCString url;
+	furi->GetSpec (url);
+
+	return url.Length() ? g_strdup (url.get()) : NULL;
 }
 
 static void
 impl_reload (EphyEmbed *embed, 
              gboolean force)
 {
-#ifndef GTKMOZEMBED_BROKEN_RELOAD
-	guint32 mflags;
-
-	mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
+	guint32 mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 
 	if (force)
 	{
@@ -514,15 +612,6 @@ impl_reload (EphyEmbed *embed,
 	}
 
 	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), mflags);
-#else
-	/* Workaround for broken reload with frames, see mozilla bug
-	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
-
-	mpriv->browser->Reload (force ? EphyBrowser::RELOAD_FORCE :
-					EphyBrowser::RELOAD_NORMAL);
-#endif
 }
 
 static void
@@ -718,15 +807,8 @@ impl_set_encoding (EphyEmbed *embed,
 		if (NS_FAILED (rv)) return;
 	}
 
-#ifndef GTKMOZEMBED_BROKEN_RELOAD
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
-#else
-	/* Workaround for broken reload with frames, see mozilla bug
-	 * http://bugzilla.mozilla.org/show_bug.cgi?id=246392
-	 */
-	mpriv->browser->Reload (EphyBrowser::RELOAD_ENCODING_CHANGE);
-#endif
 }
 
 static char *
@@ -1120,7 +1202,16 @@ _mozilla_embed_new_xul_dialog (void)
 {
 	GtkWidget *window, *embed;
 
+	g_object_ref (embed_shell);
+
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	g_object_set_data_full (G_OBJECT (window), "EmbedShellRef",
+			        embed_shell,
+				(GDestroyNotify) g_object_unref);
+	g_signal_connect_object (embed_shell, "prepare_close",
+				 G_CALLBACK (gtk_widget_destroy), window,
+				 (GConnectFlags) G_CONNECT_SWAPPED);
+
 	embed = gtk_moz_embed_new ();
 	gtk_widget_show (embed);
 	gtk_container_add (GTK_CONTAINER (window), embed);
