@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-dnd.c,v 1.17 2004/12/13 18:58:58 chpe Exp $
+ *  $Id: ephy-dnd.c,v 1.17.2.1 2005/08/13 10:49:23 chpe Exp $
  */
 
 #include "config.h"
@@ -109,7 +109,7 @@ ephy_dnd_drag_data_get (GtkWidget *widget,
 
         gtk_selection_data_set (selection_data,
                                 selection_data->target,
-                                8, result->str, result->len);
+                                8, (const guchar *) result->str, result->len);
 
 	g_string_free (result, TRUE);
 
