@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,32 +16,41 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topic-action.c,v 1.63 2004/12/13 17:57:12 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
 
-#include <gtk/gtktoolitem.h>
-#include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <string.h>
-
 #include "ephy-topic-action.h"
 #include "ephy-node-common.h"
 #include "ephy-bookmarks.h"
 #include "ephy-bookmarksbar.h"
+#include "ephy-link.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
-#include "ephy-debug.h"
 #include "ephy-dnd.h"
 #include "ephy-gui.h"
-#include "ephy-marshal.h"
+#include "ephy-debug.h"
 
-static void ephy_topic_action_init       (EphyTopicAction *action);
-static void ephy_topic_action_class_init (EphyTopicActionClass *class);
+#include <glib/gi18n.h>
+#include <gtk/gtkwidget.h>
+#include <gtk/gtkarrow.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtkmain.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <string.h>
 
 #define TOOLITEM_WIDTH_CHARS	24
 #define MENUITEM_WIDTH_CHARS	32
+#define LABEL_WIDTH_CHARS       32
 
 #define EPHY_TOPIC_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOPIC_ACTION, EphyTopicActionPrivate))
 
@@ -66,18 +76,11 @@ enum
 	PROP_TOPIC
 };
 
-enum
-{
-	OPEN,
-	OPEN_IN_TAB,
-	OPEN_IN_TABS,
-	LAST_SIGNAL
-};
+static void ephy_topic_action_class_init (EphyTopicActionClass *class);
+static void ephy_topic_action_init       (EphyTopicAction *action);
 
 static GObjectClass *parent_class = NULL;
 
-static guint ephy_topic_action_signals[LAST_SIGNAL] = { 0 };
-
 GType
 ephy_topic_action_get_type (void)
 {
@@ -98,10 +101,11 @@ ephy_topic_action_get_type (void)
 			(GInstanceInitFunc) ephy_topic_action_init,
 		};
 
-		type = g_type_register_static (GTK_TYPE_ACTION,
+		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
 					       "EphyTopicAction",
 					       &type_info, 0);
 	}
+
 	return type;
 }
 
@@ -123,6 +127,7 @@ create_tool_item (GtkAction *action)
 	button = gtk_toggle_button_new ();
 	gtk_widget_add_events (GTK_WIDGET (button), GDK_BUTTON1_MOTION_MASK);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (hbox), button);
 	g_object_set_data (G_OBJECT (item), "button", button);
@@ -163,16 +168,8 @@ menu_activate_cb (GtkWidget *item, GtkAction *action)
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 
-	if (ephy_gui_is_middle_click ())
-	{
-		g_signal_emit (action, ephy_topic_action_signals[OPEN_IN_TAB],
-			       0, location, FALSE);
-	}
-	else
-	{
-		g_signal_emit (action, ephy_topic_action_signals[OPEN],
-			       0, location);
-	}
+	ephy_link_open (EPHY_LINK (action), location, NULL,
+			ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
 }
 
 static void
@@ -285,14 +282,12 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 
 		node_list = g_list_sort (node_list, (GCompareFunc)sort_bookmarks);
 
-		for (l = g_list_first (node_list); l != NULL; l = g_list_next (l))
+		for (l = node_list; l != NULL; l = l->next)
 		{
-			EphyNode *kid;
+			EphyNode *kid = (EphyNode*) l->data;
 			const char *icon_location;
 			const char *title;
 
-			kid = (EphyNode*)l->data;
-
 			icon_location = ephy_node_get_property_string
 				(kid, EPHY_NODE_BMK_PROP_ICON);
 			title = ephy_node_get_property_string
@@ -301,8 +296,9 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 
 			item = gtk_image_menu_item_new_with_label (title);
 			label = (GtkLabel *) ((GtkBin *) item)->child;
-			gtk_label_set_max_width_chars (label, MENUITEM_WIDTH_CHARS);
+			gtk_label_set_max_width_chars (label, LABEL_WIDTH_CHARS);
 			gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+			gtk_label_set_use_underline (label, FALSE);
 
 			if (icon_location)
 			{
@@ -337,29 +333,36 @@ static void
 open_in_tabs_activate_cb (GtkWidget *item, EphyTopicAction *action)
 {
 	EphyNode *node;
-	GList *uri_list = NULL;
 	GPtrArray *children;
+	EphyTab *tab = NULL;
+	GList *node_list = NULL, *l;
 	int i;
 
 	node = g_object_get_data (G_OBJECT (item), TOPIC_NODE_DATA_KEY);
 	g_return_if_fail (node != NULL);
 
 	children = ephy_node_get_children (node);
-	for (i = 0; i < children->len; i++)
+	for (i = 0; i < children->len; ++i)
 	{
-		const char *address;
-		EphyNode *child;
-
-		child = g_ptr_array_index (children, i);
-		address = ephy_node_get_property_string
-			(child, EPHY_NODE_BMK_PROP_LOCATION);
-		uri_list = g_list_append (uri_list, g_strdup (address));
+		node_list = g_list_prepend (node_list,
+					    g_ptr_array_index (children, i));
 	}
 
-	g_signal_emit (action, ephy_topic_action_signals[OPEN_IN_TABS],
-		       0, uri_list);
-	g_list_foreach (uri_list, (GFunc)g_free, NULL);
-	g_list_free (uri_list);
+	node_list = g_list_sort (node_list, (GCompareFunc) sort_bookmarks);
+
+	for (l = node_list; l != NULL; l = l->next)
+	{
+		EphyNode *child = (EphyNode *) l->data;
+		const char *address;
+
+		address = ephy_node_get_property_string
+			(child, EPHY_NODE_BMK_PROP_LOCATION);
+
+		tab = ephy_link_open (EPHY_LINK (action), address, tab,
+				      tab ? EPHY_LINK_NEW_TAB : EPHY_LINK_NEW_WINDOW);
+	}
+
+	g_list_free (node_list);
 }
 
 static int
@@ -442,8 +445,13 @@ add_open_in_tabs_menu (EphyTopicAction *action,
 		       EphyNode *node)
 {
 	GtkWidget *item;
+	const char *label;
 
-	item = gtk_menu_item_new_with_mnemonic (_("_Open in Tabs"));
+	label = ngettext ("Open in New _Tab",
+			  "Open in New _Tabs",
+			  ephy_node_get_n_children (node));
+
+	item = gtk_menu_item_new_with_mnemonic (label);
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -686,8 +694,8 @@ button_toggled_cb (GtkWidget *button,
 		guint event_button = 0;
 
 		menu = build_menu (action);
-		g_signal_connect (menu, "deactivate",
-				  G_CALLBACK (menu_deactivate_cb), button);
+		g_signal_connect_object (menu, "deactivate",
+					 G_CALLBACK (menu_deactivate_cb), button, 0);
 
 		event = gtk_get_current_event ();
 		if (event != NULL)
@@ -749,7 +757,7 @@ show_context_menu (EphyTopicAction *action,
 		   GdkEventButton *event,
 		   GtkMenuPositionFunc func)
 {
-	GtkWidget *menu, *item;
+	GtkWidget *menu, *item, *image;
 	gboolean last;
 
 	menu = gtk_menu_new ();
@@ -761,8 +769,11 @@ show_context_menu (EphyTopicAction *action,
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Remove from Toolbar"));
 	gtk_widget_show (item);
+	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_show (image);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (remove_activate_cb), proxy);
@@ -876,7 +887,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
 	GtkWidget *button;
 
-	LOG ("connect_proxy action %p, proxy %p", action, proxy)
+	LOG ("connect_proxy action %p, proxy %p", action, proxy);
 
 	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
 
@@ -900,6 +911,16 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		g_signal_connect (button, "drag_data_delete",
 				  G_CALLBACK (drag_data_delete_cb), action);
 	}
+	else if (GTK_IS_MENU_ITEM (proxy))
+	{
+		GtkLabel *label;
+
+		label = (GtkLabel *) ((GtkBin *) proxy)->child;
+
+		gtk_label_set_use_underline (label, FALSE);
+		gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+		gtk_label_set_max_width_chars (label, LABEL_WIDTH_CHARS);
+	}
 }
 
 static void
@@ -989,40 +1010,6 @@ ephy_topic_action_class_init (EphyTopicActionClass *class)
 	object_class->set_property = ephy_topic_action_set_property;
 	object_class->get_property = ephy_topic_action_get_property;
 
-	ephy_topic_action_signals[OPEN] =
-                g_signal_new ("open",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyTopicActionClass, open),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_STRING);
-
-	ephy_topic_action_signals[OPEN_IN_TAB] =
-                g_signal_new ("open_in_tab",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyTopicActionClass, open_in_tab),
-                              NULL, NULL,
-                              ephy_marshal_VOID__STRING_BOOLEAN,
-                              G_TYPE_NONE,
-                              2,
-			      G_TYPE_STRING,
-			      G_TYPE_BOOLEAN);
-
-	ephy_topic_action_signals[OPEN_IN_TABS] =
-                g_signal_new ("open_in_tabs",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyTopicActionClass, open_in_tabs),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__POINTER,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_POINTER);
-
 	g_object_class_install_property
 		(object_class,
 		 PROP_TOPIC,
