@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-action.c,v 1.32 2005/01/08 14:02:19 chpe Exp $
+ *  $Id: ephy-location-action.c,v 1.36 2005/02/06 14:37:12 chpe Exp $
  */
 
 #include "config.h"
@@ -46,6 +46,9 @@ struct _EphyLocationActionPrivate
 	EphyBookmarks *bookmarks;
 	char *icon;
 	EphyFaviconCache *cache;
+	char *lock_stock_id;
+	char *lock_tooltip;
+	gboolean show_lock;
 };
 
 static void ephy_location_action_init       (EphyLocationAction *action);
@@ -63,9 +66,19 @@ enum
 	PROP_ADDRESS,
 	PROP_EDITABLE,
 	PROP_ICON,
+	PROP_LOCK_STOCK,
+	PROP_LOCK_TOOLTIP,
+	PROP_SHOW_LOCK,
 	PROP_WINDOW
 };
 
+enum
+{
+	LOCK_CLICKED,
+	LAST_SIGNAL
+};
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static GObjectClass *parent_class = NULL;
 
 GType
@@ -148,13 +161,20 @@ user_changed_cb (GtkWidget *proxy, EphyLocationAction *action)
 
 	address = ephy_location_entry_get_location (EPHY_LOCATION_ENTRY (proxy));
 
-	LOG ("user_changed_cb, new address %s", address)
+	LOG ("user_changed_cb, new address %s", address);
 
 	g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
 	ephy_location_action_set_address (action, address);
 	g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 }
 
+static void
+lock_clicked_cb (GtkWidget *proxy,
+		 EphyLocationAction *action)
+{
+	g_signal_emit (action, signals[LOCK_CLICKED], 0);
+}
+
 static void
 sync_address (GtkAction *gaction,
 	      GParamSpec *pspec,
@@ -163,7 +183,7 @@ sync_address (GtkAction *gaction,
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
 	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 
-	LOG ("sync_address %s", action->priv->address)
+	LOG ("sync_address %s", action->priv->address);
 
 	g_signal_handlers_block_by_func (proxy, G_CALLBACK (user_changed_cb), action);
 	ephy_location_entry_set_location (lentry, action->priv->address);
@@ -191,23 +211,52 @@ sync_icon (GtkAction *gaction,
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
 	EphyLocationActionPrivate *priv = action->priv;
 	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
-	GtkWidget *image;
 	GdkPixbuf *pixbuf;
 
-	image = ephy_location_entry_get_image (entry);
 	pixbuf = ephy_favicon_cache_get (priv->cache, priv->icon);
 
+	ephy_location_entry_set_favicon (entry, pixbuf);
+
 	if (pixbuf != NULL)
 	{
-		gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
 		g_object_unref (pixbuf);
 	}
-	else
-	{
-		gtk_image_set_from_stock (GTK_IMAGE (image),
-					  GTK_STOCK_NEW,
-					  GTK_ICON_SIZE_MENU);
-	}
+}
+
+static void
+sync_lock_stock_id (GtkAction *gaction,
+		    GParamSpec *pspec,
+		    GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_lock_stock (entry, priv->lock_stock_id);
+}
+
+static void
+sync_lock_tooltip (GtkAction *gaction,
+		   GParamSpec *pspec,
+		   GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_lock_tooltip (entry, priv->lock_tooltip);
+}
+
+static void
+sync_show_lock (GtkAction *gaction,
+	        GParamSpec *pspec,
+	        GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_show_lock (entry, priv->show_lock);
 }
 
 static char *
@@ -311,6 +360,15 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		sync_icon (action, NULL, proxy);
 		g_signal_connect_object (action, "notify::icon",
 					 G_CALLBACK (sync_icon), proxy, 0);
+		sync_lock_stock_id (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::lock-stock-id",
+					 G_CALLBACK (sync_lock_stock_id), proxy, 0);
+		sync_lock_tooltip (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::lock-tooltip",
+					 G_CALLBACK (sync_lock_tooltip), proxy, 0);
+		sync_show_lock (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::show-lock",
+					 G_CALLBACK (sync_show_lock), proxy, 0);
 
 		entry = ephy_location_entry_get_entry (lentry);
 		g_signal_connect_object (entry, "activate",
@@ -318,6 +376,8 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 					 action, 0);
 		g_signal_connect_object (proxy, "user_changed",
 					 G_CALLBACK (user_changed_cb), action, 0);
+		g_signal_connect_object (proxy, "lock-clicked",
+					 G_CALLBACK (lock_clicked_cb), action, 0);
 		g_signal_connect_object (proxy, "get-location",
 					 G_CALLBACK (get_location_cb), action, 0);
 		g_signal_connect_object (proxy, "get-title",
@@ -368,6 +428,17 @@ ephy_location_action_set_property (GObject *object,
 			g_free (action->priv->icon);
 			action->priv->icon = g_value_dup_string (value);
 			break;
+		case PROP_LOCK_STOCK:
+			g_free (action->priv->lock_stock_id);
+			action->priv->lock_stock_id = g_value_dup_string (value);
+			break;
+		case PROP_LOCK_TOOLTIP:
+			g_free (action->priv->lock_tooltip);
+			action->priv->lock_tooltip = g_value_dup_string (value);
+			break;
+		case PROP_SHOW_LOCK:
+			action->priv->show_lock = g_value_get_boolean (value);
+			break;
 		case PROP_WINDOW:
 			action->priv->window = EPHY_WINDOW (g_value_get_object (value));
 			break;
@@ -393,6 +464,15 @@ ephy_location_action_get_property (GObject *object,
 		case PROP_ICON:
 			g_value_set_string (value, action->priv->icon);
 			break;
+		case PROP_LOCK_STOCK:
+			g_value_set_string (value, action->priv->lock_stock_id);
+			break;
+		case PROP_LOCK_TOOLTIP:
+			g_value_set_string (value, action->priv->lock_tooltip);
+			break;
+		case PROP_SHOW_LOCK:
+			g_value_set_boolean (value, action->priv->show_lock);
+			break;
 		case PROP_WINDOW:
 			/* not readable */
 			break;
@@ -415,6 +495,16 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 	action_class->connect_proxy = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
 
+	signals[LOCK_CLICKED] = g_signal_new (
+		"lock-clicked",
+		EPHY_TYPE_LOCATION_ACTION,
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (EphyLocationActionClass, lock_clicked),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE,
+		0);
+
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
 					 g_param_spec_string ("address",
@@ -438,6 +528,30 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 							       NULL,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_STOCK,
+					 g_param_spec_string  ("lock-stock-id",
+							       "Lock Stock ID",
+							       "Lock Stock ID",
+							       NULL,
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_TOOLTIP,
+					 g_param_spec_string  ("lock-tooltip",
+							       "Lock Tooltip",
+							       "The icon",
+							       NULL,
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_LOCK,
+					 g_param_spec_boolean ("show-lock",
+							       "Show Lock",
+							       "Show Lock",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_WINDOW,
 					 g_param_spec_object ("window",
@@ -593,7 +707,9 @@ ephy_location_action_finalize (GObject *object)
 
 	g_list_free (priv->actions);
 	g_free (priv->address);
-	g_free (action->priv->icon);
+	g_free (priv->icon);
+	g_free (priv->lock_stock_id);
+	g_free (priv->lock_tooltip);
 	g_object_unref (priv->cache);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -613,7 +729,7 @@ ephy_location_action_set_address (EphyLocationAction *action,
 {
 	g_return_if_fail (EPHY_IS_LOCATION_ACTION (action));
 
-	LOG ("set_address %s", address)
+	LOG ("set_address %s", address);
 
 	g_free (action->priv->address);
 	action->priv->address = g_strdup (address);
