@@ -15,15 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-main.c,v 1.25.4.1 2004/11/11 12:55:55 chpe Exp $
+ *  $Id$
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-shell.h"
 #include "ephy-file-helpers.h"
+#include "ephy-object-helpers.h"
 #include "ephy-state.h"
 #include "ephy-debug.h"
 #include "ephy-stock-icons.h"
@@ -31,20 +30,23 @@
 
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-app-helper.h>
+#include <gtk/gtkaboutdialog.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gdk/gdkx.h>
 #include <libgnome/gnome-program.h>
 #include <bonobo/bonobo-main.h>
 #include <glib/gi18n.h>
-#include <glade/glade-init.h>
 #include <libgnomevfs/gnome-vfs-init.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libxml/xmlversion.h>
+#include <errno.h>
+#include <string.h>
 
 static gboolean open_in_existing = FALSE;
 static gboolean open_in_new_tab = FALSE;
 static gboolean open_fullscreen = FALSE;
 static gboolean open_as_bookmarks_editor = FALSE;
-static gboolean server_mode = FALSE;
 
 static const char *session_filename = NULL;
 static const char *bookmark_url = NULL;
@@ -70,12 +72,63 @@ static struct poptOption popt_options[] =
 	{ "bookmarks-editor", 'b', POPT_ARG_NONE, &open_as_bookmarks_editor, 0,
 	  N_("Launch the bookmarks editor"),
 	  NULL },
-	{ "server", 's', POPT_ARG_NONE, &server_mode, 0,
-	  N_("Used internally by the bonobo interface"),
-	  NULL },
 	{ NULL, 0, 0, NULL, 0, NULL, NULL }
 };
 
+/* adapted from gtk+/gdk/x11/gdkdisplay-x11.c */
+static guint32
+get_startup_id (void)
+{
+	const char *startup_id, *time_str;
+	guint32 retval = 0;
+
+	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
+	if (startup_id == NULL) return 0;
+
+	/* Find the launch time from the startup_id, if it's there.  Newer spec
+	* states that the startup_id is of the form <unique>_TIME<timestamp>
+	*/
+	time_str = g_strrstr (startup_id, "_TIME");
+	if (time_str != NULL)
+	{
+		gulong value;
+		gchar *end;
+		errno = 0;
+	
+		/* Skip past the "_TIME" part */
+		time_str += 5;
+	
+		value = strtoul (time_str, &end, 0);
+		if (end != time_str && errno == 0)
+		{
+			retval = (guint32) value;
+		}
+	}
+
+	return retval;
+}
+
+static void
+handle_url (GtkAboutDialog *about,
+	    const char *link,
+	    gpointer data)
+{
+	ephy_shell_new_tab (ephy_shell, NULL, NULL, link,
+			    EPHY_NEW_TAB_OPEN_PAGE);
+}
+
+static void
+handle_email (GtkAboutDialog *about,
+	      const char *link,
+	      gpointer data)
+{
+	char *address;
+
+	address = g_strdup_printf ("mailto:%s\n", link);
+	gnome_vfs_url_show (address);
+	g_free (address);
+}
+
 static void
 shell_weak_notify (gpointer data,
                    GObject *where_the_object_was)
@@ -83,11 +136,55 @@ shell_weak_notify (gpointer data,
 	gtk_main_quit ();
 }
 
-static gboolean
-idle_unref (GObject *object)
+/* Copied from libnautilus/nautilus-program-choosing.c; Needed in case
+ * we have no DESKTOP_STARTUP_ID (with its accompanying timestamp).
+ */
+static Time
+slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 {
-	g_object_unref (object);
-	return FALSE;
+	Window xwindow;
+	XEvent event;
+	
+	{
+		XSetWindowAttributes attrs;
+		Atom atom_name;
+		Atom atom_type;
+		char* name;
+		
+		attrs.override_redirect = True;
+		attrs.event_mask = PropertyChangeMask | StructureNotifyMask;
+		
+		xwindow =
+			XCreateWindow (xdisplay,
+				       RootWindow (xdisplay, 0),
+				       -100, -100, 1, 1,
+				       0,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CWOverrideRedirect | CWEventMask,
+				       &attrs);
+		
+		atom_name = XInternAtom (xdisplay, "WM_NAME", TRUE);
+		g_assert (atom_name != None);
+		atom_type = XInternAtom (xdisplay, "STRING", TRUE);
+		g_assert (atom_type != None);
+		
+		name = "Fake Window";
+		XChangeProperty (xdisplay, 
+				 xwindow, atom_name,
+				 atom_type,
+				 8, PropModeReplace, name, strlen (name));
+	}
+	
+	XWindowEvent (xdisplay,
+		      xwindow,
+		      PropertyChangeMask,
+		      &event);
+	
+	XDestroyWindow(xdisplay, xwindow);
+	
+	return event.xproperty.time;
 }
 
 int
@@ -98,6 +195,7 @@ main (int argc, char *argv[])
 	GnomeProgram *program;
 	EphyShellStartupFlags startup_flags;
 	const char **args, *string_arg;
+	guint32 user_time;
 	gboolean new_instance;
 	GError *err = NULL;
 
@@ -113,6 +211,9 @@ main (int argc, char *argv[])
 	 */
 	LIBXML_TEST_VERSION
 
+	/* get this early, since gdk will unset the env var */
+	user_time = get_startup_id ();
+
 	program = gnome_program_init (PACKAGE, VERSION,
                                       LIBGNOMEUI_MODULE, argc, argv,
                                       GNOME_PARAM_POPT_TABLE, popt_options,
@@ -120,9 +221,16 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
                                       NULL);
 
+	/* Get a timestamp manually if need be */
+	if (user_time == 0)
+		user_time = slowly_and_stupidly_obtain_timestamp (gdk_display);
+
 	g_set_application_name (_("Epiphany Web Browser"));
 
-        g_object_get_property (G_OBJECT (program),
+	/* Set default window icon */
+	gtk_window_set_default_icon_name ("web-browser");
+
+	g_object_get_property (G_OBJECT (program),
                                GNOME_PARAM_POPT_CONTEXT,
                                g_value_init (&context_as_value, G_TYPE_POINTER));
         context = g_value_get_pointer (&context_as_value);
@@ -161,13 +269,8 @@ main (int argc, char *argv[])
 		startup_flags |= EPHY_SHELL_STARTUP_ADD_BOOKMARK;
 		string_arg = bookmark_url;
 	}
-	else if (server_mode)
-	{
-		startup_flags |= EPHY_SHELL_STARTUP_SERVER;
-	}
 
 	gnome_vfs_init ();
-	glade_gnome_init ();
 	ephy_debug_init ();
 	ephy_file_helpers_init ();
 	ephy_stock_icons_init ();
@@ -175,10 +278,16 @@ main (int argc, char *argv[])
 	eel_gconf_monitor_add ("/apps/epiphany/lockdown");
 	eel_gconf_monitor_add ("/desktop/gnome/lockdown");
 
+	/* Extensions may want these, so don't initialize in window-cmds */
+	gtk_about_dialog_set_url_hook (handle_url, NULL, NULL);
+	gtk_about_dialog_set_email_hook (handle_email, NULL, NULL);
+
 	bonobo_activate ();
 
-	ephy_shell = ephy_shell_new ();
+	ephy_shell_new ();
+	g_assert (ephy_shell != NULL);
 	new_instance = ephy_shell_startup (ephy_shell, startup_flags,
+					   user_time,
 					   args, string_arg, &err);
 
 	if (err != NULL)
@@ -190,10 +299,10 @@ main (int argc, char *argv[])
 						GTK_MESSAGE_ERROR, err->message);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 	}
-	else if (new_instance)
+	else if (new_instance && ephy_shell)
 	{
 		g_object_weak_ref (G_OBJECT (ephy_shell), shell_weak_notify, NULL);
-		g_idle_add ((GSourceFunc) idle_unref, ephy_shell);
+		ephy_object_idle_unref (ephy_shell);
 
 		gtk_main ();
 	}
@@ -205,6 +314,7 @@ main (int argc, char *argv[])
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
 	gnome_vfs_shutdown ();
+	xmlCleanupParser ();
 	poptFreeContext (context);
 
 	return 0;
