@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.118 2005/02/02 19:32:47 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
@@ -40,7 +40,6 @@
 #include "ephy-session.h"
 #include "downloader-view.h"
 #include "egg-toolbars-model.h"
-#include "eggtypebuiltins.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-toolbar.h"
 #include "ephy-automation.h"
@@ -73,7 +72,6 @@ struct _EphyShellPrivate
 	EphySession *session;
 	EphyBookmarks *bookmarks;
 	EggToolbarsModel *toolbars_model;
-	guint toolbar_style_notifier_id;
 	EggToolbarsModel *fs_toolbars_model;
 	EphyExtensionsManager *extensions_manager;
 	GObject *dbus_service;
@@ -474,8 +472,13 @@ ephy_shell_startup (EphyShell *shell,
 				       "from Bonobo when attempting to locate the automation "
 				       "object."));
 			automation = NULL;
+			goto done;
 		}
-		else if (flags & EPHY_SHELL_STARTUP_BOOKMARKS_EDITOR)
+
+		/* init the session manager up here so we can quit while the resume dialogue is on */
+		gnome_session_init (shell);
+
+		if (flags & EPHY_SHELL_STARTUP_BOOKMARKS_EDITOR)
 		{
 			GNOME_EphyAutomation_openBookmarksEditorWithStartupId
 				(automation, user_time, &ev);
@@ -503,14 +506,10 @@ ephy_shell_startup (EphyShell *shell,
 				   flags & EPHY_SHELL_STARTUP_FULLSCREEN);
 		}
 
-		if (automation)
-		{
-			bonobo_object_release_unref (automation, &ev);
-		}
-
-		gnome_session_init (shell);
+		bonobo_object_release_unref (automation, &ev);
 	}
 
+done:
 	CORBA_exception_free (&ev);
 	gdk_notify_startup_complete ();
 
@@ -547,11 +546,6 @@ ephy_shell_finalize (GObject *object)
 
 	LOG ("Unref toolbars model");
 	if (shell->priv->toolbars_model)
-	if (shell->priv->toolbar_style_notifier_id != 0)
-	{
-		eel_gconf_notification_remove (shell->priv->toolbar_style_notifier_id);
-	}
-	if (shell->priv->toolbars_model)
 	{
 		g_object_unref (shell->priv->toolbars_model);
 	}
@@ -631,10 +625,25 @@ ephy_shell_new (void)
 	return EPHY_SHELL (g_object_new (EPHY_TYPE_SHELL, NULL));
 }
 
-static void
+static gboolean
+url_is_empty (const char *location)
+{
+	gboolean is_empty = FALSE;
+
+        if (location == NULL || location[0] == '\0' ||
+            strcmp (location, "about:blank") == 0)
+        {
+                is_empty = TRUE;
+        }
+
+        return is_empty;
+}
+
+static gboolean
 load_homepage (EphyEmbed *embed)
 {
 	char *home;
+	gboolean is_empty;
 
 	home = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
 
@@ -645,9 +654,13 @@ load_homepage (EphyEmbed *embed)
 		home = g_strdup ("about:blank");
 	}
 
+	is_empty = url_is_empty (home);
+
 	ephy_embed_load_url (embed, home);
 
 	g_free (home);
+
+	return is_empty;
 }
 
 /**
@@ -681,6 +694,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	EphyEmbed *previous_embed = NULL;
 	GtkWidget *nb;
 	int position = -1;
+	gboolean is_empty = FALSE;
 	EphyToolbar *toolbar;
 
 	if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
@@ -724,19 +738,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
 	ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
 
-	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
-	    flags & EPHY_NEW_TAB_NEW_PAGE)
-	{
-		ephy_tab_set_location (tab, "", EPHY_TAB_ADDRESS_EXPIRE_NEXT);
-		ephy_toolbar_activate_location (toolbar);
-		load_homepage (embed);
-	}
-	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
-	{
-		g_assert (url != NULL);
-		ephy_embed_load_url (embed, url);
-	}
-
 	if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0)
 	{
 		gtk_widget_show (GTK_WIDGET (window));
@@ -747,6 +748,41 @@ ephy_shell_new_tab_full (EphyShell *shell,
 		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
 
+	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
+	    flags & EPHY_NEW_TAB_NEW_PAGE)
+	{
+		ephy_tab_set_location (tab, "", EPHY_TAB_ADDRESS_EXPIRE_NEXT);
+		ephy_toolbar_activate_location (toolbar);
+		is_empty = load_homepage (embed);
+	}
+	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
+	{
+		g_assert (url != NULL);
+		ephy_embed_load_url (embed, url);
+		is_empty = url_is_empty (url);
+	}
+
+        /* Make sure the initial focus is somewhere sensible and not, for
+         * example, on the reload button.
+         */
+        if (in_new_window || jump_to)
+        {
+                /* If the location entry is blank, focus that, except if the
+                 * page was a copy */
+                if (is_empty)
+                {
+                        /* empty page, focus location entry */
+			toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
+			ephy_toolbar_activate_location (toolbar);
+                }
+                else if (embed != NULL) 
+                {
+			/* non-empty page, focus the page. but make sure the widget is realised first! */
+			gtk_widget_realize (GTK_WIDGET (embed));
+			ephy_embed_activate (embed);
+                }
+        }
+
 	return tab;
 }
 
@@ -813,46 +849,6 @@ ephy_shell_get_bookmarks (EphyShell *shell)
 	return shell->priv->bookmarks;
 }
 
-static void
-toolbar_style_notifier (GConfClient *client,
-			guint cnxn_id,
-			GConfEntry *entry,
-			EphyShell *shell)
-{
-	EggToolbarsModel *model = shell->priv->toolbars_model;
-	GFlagsClass *flags_class;
-	const GFlagsValue *value;
-	EggTbModelFlags flags, new_flags = 0;
-	char *pref;
-	int i, n_toolbars;
-
-	g_return_if_fail (model != NULL);
-
-	pref = eel_gconf_get_string (CONF_INTERFACE_TOOLBAR_STYLE);
-	if (pref != NULL)
-	{
-		flags_class = g_type_class_ref (EGG_TYPE_TB_MODEL_FLAGS);
-		value = g_flags_get_value_by_nick (flags_class, pref);
-		if (value != NULL)
-		{
-			new_flags = value->value;
-		}
-		g_type_class_unref (flags_class);
-	}
-	new_flags &= EGG_TB_MODEL_STYLES_MASK;
-
-	n_toolbars = egg_toolbars_model_n_toolbars (model);
-	for (i = 0; i < n_toolbars; i++)
-	{
-		flags = egg_toolbars_model_get_flags (model, i);
-		flags &= ~EGG_TB_MODEL_STYLES_MASK;
-		flags |= new_flags;
-		egg_toolbars_model_set_flags (model, i, new_flags);
-	}
-
-	g_free (pref);
-}
-
 GObject *
 ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 {
@@ -895,12 +891,6 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 			/* ok, now we can load the model */
 			ephy_toolbars_model_load
 				(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
-
-			toolbar_style_notifier (NULL, 0, NULL, shell);
-			shell->priv->toolbar_style_notifier_id = eel_gconf_notification_add
-				(CONF_INTERFACE_TOOLBAR_STYLE,
-				 (GConfClientNotifyFunc) toolbar_style_notifier, shell);
-
 		}
 
 		return G_OBJECT (shell->priv->toolbars_model);
