@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.h,v 1.32 2005/01/15 16:47:39 crispin Exp $
+ *  $Id: ephy-shell.h,v 1.34 2005/02/02 19:32:47 chpe Exp $
  */
 
 #ifndef EPHY_SHELL_H
@@ -74,13 +74,14 @@ typedef enum
 
 	/* Page mode */
 	EPHY_NEW_TAB_FULLSCREEN_MODE	= 1 << 4,
+	EPHY_NEW_TAB_DONT_SHOW_WINDOW	= 1 << 5,
 
 	/* Tabs */
 	EPHY_NEW_TAB_APPEND_LAST	= 1 << 7,
 	EPHY_NEW_TAB_APPEND_AFTER	= 1 << 8,
 	EPHY_NEW_TAB_JUMP		= 1 << 9,
 	EPHY_NEW_TAB_IN_NEW_WINDOW	= 1 << 10,
-	EPHY_NEW_TAB_IN_EXISTING_WINDOW	= 1 << 11
+	EPHY_NEW_TAB_IN_EXISTING_WINDOW	= 1 << 11,
 } EphyNewTabFlags;
 
 struct _EphyShell
@@ -112,6 +113,7 @@ EphyShell      *ephy_shell_new				(void);
 
 gboolean	ephy_shell_startup			(EphyShell *shell,
 							 EphyShellStartupFlags flags,
+							 guint32 user_time,
 							 const char **args,
 							 const char *string_arg,
 							 GError **error);
@@ -122,6 +124,14 @@ EphyTab	       *ephy_shell_new_tab			(EphyShell *shell,
 							 const char *url,
 							 EphyNewTabFlags flags);
 
+EphyTab	       *ephy_shell_new_tab_full			(EphyShell *shell,
+							 EphyWindow *parent_window,
+							 EphyTab *previous_tab,
+							 const char *url,
+							 EphyNewTabFlags flags,
+							 EphyEmbedChrome chrome,
+							 guint32 user_time);
+
 GObject	       *ephy_shell_get_session			(EphyShell *shell);
 
 EphyBookmarks  *ephy_shell_get_bookmarks		(EphyShell *shell);
