@@ -15,16 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.h,v 1.31 2004/10/22 21:43:02 chpe Exp $
+ *  $Id: ephy-window.h,v 1.34 2005/01/08 16:32:09 chpe Exp $
  */
 
 #ifndef EPHY_WINDOW_H
 #define EPHY_WINDOW_H
 
-#include "ephy-notebook.h"
 #include "ephy-embed.h"
 #include "ephy-tab.h"
-#include "ephy-dialog.h"
 
 #include <glib-object.h>
 #include <glib.h>
@@ -89,10 +87,6 @@ void		  ephy_window_jump_to_tab	  (EphyWindow *window,
 void		  ephy_window_load_url		  (EphyWindow *window,
 						   const char *url);
 
-void		  ephy_window_load_in_tabs	  (EphyWindow *window,
-						   EphyTab *first_tab,
-						   char **uris);
-
 void		  ephy_window_set_zoom		  (EphyWindow *window,
 						   float zoom);
 
@@ -106,8 +100,6 @@ GList		 *ephy_window_get_tabs		  (EphyWindow *window);
 
 void		  ephy_window_find		  (EphyWindow *window);
 
-void		  ephy_window_print		  (EphyWindow *window);
-
 G_END_DECLS
 
 #endif
