@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs-dialog.c,v 1.103 2004/12/19 14:19:03 chpe Exp $
+ *  $Id: prefs-dialog.c,v 1.108.2.1 2005/07/29 22:28:29 chpe Exp $
  */
 
 #include "config.h"
@@ -56,6 +56,7 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtknotebook.h>
 #include <string.h>
 
 #define CONF_FONTS_FOR_LANGUAGE	"/apps/epiphany/dialogs/preferences_font_language"
@@ -687,7 +688,7 @@ fonts_language_changed_cb (EphyDialog *dialog,
 
 	code = g_value_get_string (value);
 
-	LOG ("fonts language combo changed, new code '%s'", code)
+	LOG ("fonts language combo changed, new code '%s'", code);
 
 	setup_font_combo (dialog, "variable", code, VARIABLE_PROP);
 	setup_font_combo (dialog, "monospace", code, MONOSPACE_PROP);
@@ -739,7 +740,7 @@ create_fonts_language_menu (EphyDialog *dialog)
 	{
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_FONTS_LANG_NAME, _(fonts_languages[i].title),
+				    COL_FONTS_LANG_NAME, Q_(fonts_languages[i].title),
 				    COL_FONTS_LANG_CODE, fonts_languages[i].code,
 				    -1);
 	}
@@ -1006,7 +1007,9 @@ get_name_for_lang_code (PrefsDialog *pd,
 
 		if (localename != NULL)
 		{
-			/* translators: the first %s is the language name, and the
+			/* Translators: The text before the "|" is context to help you decide on
+			 * the correct translation. You MUST OMIT it in the translated string.
+			 * Translators: the first %s is the language name, and the
 			 * second %s is the locale name. Example:
 			 * "French (France)"
 			 */
@@ -1016,13 +1019,17 @@ get_name_for_lang_code (PrefsDialog *pd,
 		}
 		else
 		{
+			/* Translators: The text before the "|" is context to help you decide on
+			 * the correct translation. You MUST OMIT it in the translated string. */
 			name = g_strdup_printf (Q_("language|%s (%s)"),
 						dgettext (ISO_639_DOMAIN, langname), str[1]);
 		}
 	}
 	else
 	{
-		/* translators: this refers to a user-define language code
+		/* Translators: The text before the "|" is context to help you decide on
+		 * the correct translation. You MUST OMIT it in the translated string.
+		 * Translators: this refers to a user-define language code
 		 * (one which isn't in our built-in list).
 		 */
 		name = g_strdup_printf (Q_("language|User defined (%s)"), code);
@@ -1169,7 +1176,7 @@ language_editor_add_button_clicked_cb (GtkWidget *button,
 
 		g_object_add_weak_pointer
 			(G_OBJECT (pd->priv->add_lang_dialog),
-			(gpointer *) &pd->priv->add_lang_dialog);;
+			(gpointer *) &pd->priv->add_lang_dialog);
 	}
 
 	ephy_dialog_show (pd->priv->add_lang_dialog);
@@ -1420,7 +1427,7 @@ create_language_section (EphyDialog *dialog)
 }
 
 static char*
-get_download_button_label ()
+get_download_button_label (void)
 {
 	char *key, *label, *downloads_path, *converted_dp;
 
@@ -1581,14 +1588,18 @@ void
 prefs_homepage_current_button_clicked_cb (GtkWidget *button,
 					  EphyDialog *dialog)
 {
-	GtkWidget *parent;
+	EphySession *session;
+	EphyWindow *window;
 	EphyEmbed *embed;
 	char *location;
 
-	parent = ephy_dialog_get_parent (dialog);
-	g_return_if_fail (parent != NULL);
+	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+	window = ephy_session_get_active_window (session);
 
-	embed = ephy_window_get_active_embed (EPHY_WINDOW (parent));
+	/* can't do anything in this case */
+	if (window == NULL) return;
+
+	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
 	location = ephy_embed_get_location (embed, TRUE);
